// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Hannah Bast <bast>.

#include "./CsvParser.h"
#include <assert.h>
#include <string>
#include <vector>

using std::cout;
using std::endl;
using std::vector;
using std::string;

// ____________________________________________________________________________
void CsvParser::openFile(string fileName)
{
  _fileStream.open(fileName.c_str());
  assert(_fileStream.good());
  readNextLine();
  // remove utf 8 bom if existing
  if (_currentLine[0] == -17 && _currentLine[1] == -69 &&
      _currentLine[2] == -65)
  {
    _currentLine = _currentLine.substr(3);
    _currentItems[0] = _currentItems[0].substr(3);
  }
  assert(_currentItems.size() > 0);
}

// ____________________________________________________________________________
void CsvParser::readNextLine()
{
  assert(_fileStream.good());
  getline(_fileStream, _currentLine);
  _currentItems.clear();
  // remove windows line return if present
  if (_currentLine[_currentLine.size() - 1] == '\r')
    _currentLine.erase(_currentLine.size() - 1);
  if (_fileStream.eof() && _currentLine.empty()) return;


  // go through string, finding commas that separate items in csv-file
  // when finding an opening double-quote, try to find the closing double-quote,
  // then add the item without the enclosing quotation mark!
  size_t posCurrentItem = 0;
  size_t posDoubleQuote;
  size_t posComma;
  bool doubleQuoteFound;
  while (posCurrentItem != string::npos)
  {
    doubleQuoteFound = false;
    // printf("looking at %s\n", _currentLine.c_str() + posCurrentItem);
    posComma = _currentLine.find(',', posCurrentItem);
    posDoubleQuote = _currentLine.find('"', posCurrentItem);
    // check if there has been a quotation mark before the comma!
    // first check unnecessary, just for code clarity :)
    if (posDoubleQuote != string::npos && (posDoubleQuote < posComma))
    {
      doubleQuoteFound = true;
      // now we have to assume the found double quote is a double quote
      // enclosing a csv-field...therefore find the closing double quote
      // which should be just one double-quote, not two subsequent
      // double-quotes (see http://tools.ietf.org/html/rfc4180#page-2)
      bool closingDoubleQuoteFound = false;
      while (!closingDoubleQuoteFound)
      {
        posDoubleQuote = _currentLine.find('"', posDoubleQuote + 1);
        // now check whether this is really just a single double quote!
        if (_currentLine[posDoubleQuote + 1] != '"')
          closingDoubleQuoteFound = true;
      }
      posComma =_currentLine.find(',', posDoubleQuote);
    }
    if (doubleQuoteFound)
    {
      // strip double quotes at start and end...
      //_currentLine[posDoubleQuote] = '\0';
      if (posComma != string::npos)
        _currentItems.push_back(_currentLine.substr(posCurrentItem + 1,
                                               posComma - posCurrentItem - 2));
      else
        _currentItems.push_back(_currentLine.substr(posCurrentItem + 1,
                                     _currentLine.size() - posCurrentItem - 2));

    }
    else
    {
      // check if this was the last item...
      if (posComma != string::npos)
      {
        //printf("pos comma is %zu from %zu\n", posComma, _currentLine.size());
      }
      _currentItems.push_back(_currentLine.substr(posCurrentItem,
                                                  posComma - posCurrentItem));
    }
    if (posComma != string::npos)
     posCurrentItem = posComma + 1;
    else
      posCurrentItem = string::npos;
  }
}

void CsvParser::readNextNonEmptyLine()
{
  // always read once, then continue reading as long as current line has size 0
  readNextLine();
  while (!_fileStream.eof() && _currentLine.size() == 0)
  {
    readNextLine();
  }
}
// ____________________________________________________________________________
const string CsvParser::getItem(size_t i)
{
  assert(i < _currentItems.size());
  return _currentItems[i];
}

// ____________________________________________________________________________
void CsvParser::closeFile()
{
  assert(_fileStream.is_open());
  _fileStream.close();
}
