/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

#include "./RoadNetwork.h"
#include <stdio.h>
#include <map>
#include <iostream>
#include <sstream>
#include <limits>
#include <string>
#include <vector>
#include <algorithm>
#include <set>
#include <queue>
#include "./Edge.h"
#include"./CsvParser.h"

using std::map;
using std::cout;
using std::endl;
using std::queue;
using std::set;
unsigned int randomSeed;
Graph::Graph()
{
  this->_nodes = new vector<NodeExpanded>();
  this->_outgoingEdges = new vector<vector<Edge> >();
  this->_incomingEdges = new vector<vector<Edge> >();
}

Graph::Graph(const Graph& orig)
{
  this->_nodes = new vector<NodeExpanded>();
  this->_outgoingEdges = new vector<vector<Edge> >();
  this->_incomingEdges = new vector<vector<Edge> >();
  this->_nodes->insert(this->_nodes->begin(),
                       orig._nodes->begin(), orig._nodes->end());
  this->_outgoingEdges->insert(this->_outgoingEdges->begin(),
                      orig._outgoingEdges->begin(), orig._outgoingEdges->end());
  this->_incomingEdges->insert(this->_incomingEdges->begin(),
                      orig._incomingEdges->begin(), orig._incomingEdges->end());
}

Graph::~Graph()
{
  if (this->_nodes != NULL)
    delete _nodes;
  _nodes = NULL;
  if (this->_outgoingEdges != NULL)
    delete _outgoingEdges;
  _outgoingEdges = NULL;
  if (this->_incomingEdges != NULL)
    delete _incomingEdges;
  _incomingEdges = NULL;
}
void Graph::addNode(NodeExpanded node)
{
  assert(static_cast<size_t>(node.id) == _nodes->size());
  _nodes->push_back(node);
  _outgoingEdges->push_back(vector<Edge>());
  _incomingEdges->push_back(vector<Edge>());
}
void Graph::addDummyNode(NodeExpanded node)
{
  _nodes->push_back(node);
}
void Graph::removeDummyNode()
{
  _nodes->erase(_nodes->end());
}
void Graph::addEdge(int startId, int endId, int time)
{
  if (static_cast<size_t>(startId) >= _nodes->size())
  {
    printf("Error: startId %d outside of range (graph has %zu nodes)\n",
            startId, _nodes->size());
  }
  if (static_cast<size_t>(endId) >= _nodes->size())
  {
    printf("Error endId %d outside of range (graph has %zu nodes)\n",
            endId, _nodes->size());
  }
  if (static_cast<size_t>(startId) >= _outgoingEdges->size())
  {
    printf("Error startid %d outside of outedge range (adjacency lists have "
            "%zu lists)\n",
            startId, _outgoingEdges->size());
  }
  if (static_cast<size_t>(endId) >= _incomingEdges->size())
  {
    printf("Error endid %d outside of inedge range (adjacency lists have "
            "%zu lists)\n",
            endId, _incomingEdges->size());
  }

//  NodeExpanded* startNode = getNode(startId);
//  NodeExpanded* endNode = getNode(endId);
//
//  if (time < 1)
//    std::cout << "adding edge with time " << time
//              << "from "<< startNode->latitude << " / "
//              << startNode->longitude
//              << " to " << endNode->latitude
//              << " / " << endNode->longitude << "\n";
  _outgoingEdges->at(startId).push_back(Edge(endId, time));
  _incomingEdges->at(endId).push_back(Edge(startId, time));
}


NodeExpanded* Graph::getNode(int id) const
{
  return &((*_nodes)[id]);
}

vector<Edge>* Graph::getInEdges(int nodeId) const
{
  return &((*_incomingEdges)[nodeId]);
}

vector<Edge>* Graph::getOutEdges(int nodeId) const
{
  return &((*_outgoingEdges)[nodeId]);
}

size_t Graph::getNumNodes() const
{
  return _nodes->size();
}

void Graph::reduceToBiggestComponent()
{
  printf("cleaning graph...");
  // find biggest connected Component
  vector<size_t> biggestSubgraph;
  findBiggestConnectedSubgraph(&biggestSubgraph);
  // remove all nodes not part of biggest connected subgraph and at the same
  // time update ids to new ids...
  vector<NodeExpanded> tempNodeVector;
  vector<vector<Edge> > tempOutVector;
  vector<vector<Edge> > tempInVector;
  map<int, int> idMap;
  NodeExpanded* nextValidNode;
  size_t nextValidId;
  for (size_t i = 0; i < biggestSubgraph.size(); i++)
  {
    nextValidId = biggestSubgraph[i];
    nextValidNode = getNode(nextValidId);
    nextValidNode->id = i;
    tempNodeVector.push_back(*nextValidNode);
    tempOutVector.push_back(*getOutEdges(nextValidId));
    tempInVector.push_back(*getInEdges(nextValidId));
    idMap[nextValidId] = i;
  }
  this->_nodes->clear();
  this->_nodes->insert(this->_nodes->begin(), tempNodeVector.begin(),
                           tempNodeVector.end());
  this->_incomingEdges->clear();
  this->_incomingEdges->insert(this->_incomingEdges->begin(),
                               tempInVector.begin(), tempInVector.end());
  this->_outgoingEdges->clear();
  this->_outgoingEdges->insert(this->_outgoingEdges->begin(),
                               tempOutVector.begin(), tempOutVector.end());
  assert(this->_nodes->size() == biggestSubgraph.size());
  // now go through edges and update ids...
  vector<Edge>* inEdges;
  vector<Edge>* outEdges;
  printf("\n updating ids....");
  for (size_t nodeId = 0; nodeId < _nodes->size(); nodeId++)
  {
    inEdges = getInEdges(nodeId);
    for (size_t i = 0; i < inEdges->size(); i++)
    {
      assert(idMap.count(inEdges->at(i).otherNodeId) != 0);
      inEdges->at(i).otherNodeId = idMap[inEdges->at(i).otherNodeId];
    }
    outEdges = getOutEdges(nodeId);
    for (size_t i = 0; i < outEdges->size(); i++)
    {
      assert(idMap.count(outEdges->at(i).otherNodeId) != 0);
      outEdges->at(i).otherNodeId = idMap[outEdges->at(i).otherNodeId];
    }
  }
  printf(" done.\n");
}

// _____________________________________________________________________________
NodeExpanded* Graph::getRandomNodeWithNeighbours()
{
  size_t idToTry = rand_r(&randomSeed) % getNumNodes();
  NodeExpanded* nodeToTry = getNode(idToTry);
  // check that node has neighbours
  bool hasNeighbours = getInEdges(idToTry)->size() > 0 &&
                       getOutEdges(idToTry)->size() > 0;
  if (hasNeighbours)
    return nodeToTry;
  else
    return getRandomNodeWithNeighbours();
}

// _____________________________________________________________________________
int Graph::findIdOfClosestNode(double latitude, double longitude)
{
  double minDistance = std::numeric_limits<double>::max();
  // construct tempnode for distance calculations..
  NodeExpanded tempNode = NodeExpanded(-1, latitude, longitude);
  int closestNodeId = -1;
  double distance;
  for (size_t nodeId = 0; nodeId < _nodes->size(); nodeId++)
  {
    distance = tempNode.straightLineDistanceTo((*_nodes)[nodeId]);
    if (distance < minDistance)
    {
      minDistance = distance;
      closestNodeId = nodeId;
    }
  }
  return closestNodeId;
}

void Graph::findBiggestConnectedSubgraph(
                                       vector<size_t>* biggestConnectedSubGraph)
{
  set<size_t> alreadyReachedNodes;
  vector<size_t> maxSubgraph;
  // go through all nodes, always find all nodes they are connected with...
  for (size_t nodeId = 0; nodeId < this->_nodes->size(); nodeId++)
  {
    if (alreadyReachedNodes.count(nodeId) != 0) continue;
    vector<size_t> nextSubgraph;

    NodeExpanded* curNode = getNode(nodeId);
    queue<NodeExpanded*> subgraphNodes;  // reached nodes in this subgraph
    subgraphNodes.push(curNode);
    alreadyReachedNodes.insert(curNode->id);

    while (!subgraphNodes.empty())
    {
      curNode = subgraphNodes.front();
      nextSubgraph.push_back(curNode->id);
      subgraphNodes.pop();
      NodeExpanded* newNode;
      vector<Edge>* outEdges = getOutEdges(curNode->id);
      for (size_t i = 0; i < outEdges->size(); i++)
      {
        newNode = getNode(outEdges->at(i).otherNodeId);
        // only insert if not in reached nodes already
        if (alreadyReachedNodes.count(newNode->id) == 0)
        {
          subgraphNodes.push(newNode);
          alreadyReachedNodes.insert(newNode->id);
        }
      }
      vector<Edge>* inEdges = getInEdges(curNode->id);
      for (size_t i = 0; i < inEdges->size(); i++)
      {
        newNode = getNode(inEdges->at(i).otherNodeId);
        if (alreadyReachedNodes.count(newNode->id) == 0)
        {
          subgraphNodes.push(newNode);
          alreadyReachedNodes.insert(newNode->id);
        }
      }
    }
    // check if bigger subgraph found...
    if (maxSubgraph.size() < nextSubgraph.size())
      maxSubgraph = nextSubgraph;
  }

  // now copy the max subgraph...
  biggestConnectedSubGraph->clear();
  biggestConnectedSubGraph->insert(biggestConnectedSubGraph->begin(),
                                  maxSubgraph.begin(), maxSubgraph.end());
  sort(biggestConnectedSubGraph->begin(), biggestConnectedSubGraph->end());
}

