#include <getopt.h>
#include <iostream>
#include <string>
#include <vector>
#include "./EdgeDependent.h"
//#include "./HybridTransitRoadNetwork.h"
#include "./MultiCriteriaHybridDijkstra.h"
#include "./Timer.h"
#include "./DependentDijkstra.h"

void printUsage()
{
  printf("This program performs shortest path queries on dependent "
         "transit networks.\n\n"
         "USAGE: ./TransitNetworkDependentMain [OPTIONS] GTFSFolder OSMFile\n");
}

using std::string;
using std::vector;
int main(int argc, char** argv)
{
  string GTFSFolder;
  string OSMFile;
  // Needed for getopt_long.
  int optChr;
  optind  = 1;
  while (true)
  {
    int optionIndex = 0;
    // Define command-line options.
    static struct option longOptions[] =
    {
      {"help", no_argument, 0, 'h'},
      {0, 0, 0, 0}
    };

    optChr = getopt_long(argc, argv, "h", longOptions, &optionIndex);

    if (optChr == -1) break;

    switch (optChr)
    {
      case 0:
        break;
      case 'h':
        printUsage();
        exit(0);
      default:
        printf("unknown option: %c\n", optChr);
        printUsage();
        return 1;
        break;
    }
  }

  // check that GTFS folder was given
  if (argc != 3)
  {
    printf("Please specify GTFS and OSM input folder!\n\n");
    printUsage();
    return 1;
  }
  GTFSFolder = argv[1];
  OSMFile = argv[2];
  // parse transit network
  TransitNetworkDependent transitNetwork;
  TransitNetworkExpanded transitNetwork1;
  RoadNetwork roadNetwork;
  roadNetwork.readFromOsmFile(OSMFile);
  roadNetwork.reduceToBiggestComponent();
  transitNetwork.parseFromGTFSFiles(GTFSFolder);
  transitNetwork1.parseFromGTFSFiles(GTFSFolder,0 , 4*60);
  HybridTransitRoadNetwork hybridNetwork(&roadNetwork, NULL, &transitNetwork1);
  hybridNetwork.buildHybridNetwork(4);
  MultiCriteriaHybridDijkstra routePlanner(&hybridNetwork);
  //printf("Nodes in Transit Network: %zu\n", hybridNetwork.getNumNodes());
  // TODO(Robin): also allow direct use(?)...
//  while (true)
//  {
//    printf("Please enter the beginning letters for the departure stop: ");
//    string stopPrefix;
//    vector<string> possibleStops;
//    while (possibleStops.size() == 0)
//    {
//      std::cin >> stopPrefix;
//      hybridNetwork.getStopsStartingWith(stopPrefix, &possibleStops);
//      if (possibleStops.size() == 0)
//        printf("No stops found starting with %s\n", stopPrefix.c_str());
//    }
//    printf("\nChoose your departure stop:\n");
//    for (size_t i = 0; i < possibleStops.size(); i++)
//    {
//      printf("%3zu - %s\n", i, possibleStops[i].c_str());
//    }
//    int departureStopNr;
//    std::cin >> departureStopNr;
//    string departureStopName = possibleStops[departureStopNr];
//    printf("Please enter the beginning letters for the destination stop: ");
//    possibleStops.clear();
//    stopPrefix = "";
//    while (possibleStops.size() == 0)
//    {
//      std::cin >> stopPrefix;
//      hybridNetwork.getStopsStartingWith(stopPrefix, &possibleStops);
//      if (possibleStops.size() == 0)
//        printf("No stops found starting with %s\n", stopPrefix.c_str());
//    }
//    printf("\nChoose your destination stop:\n");
//    for (size_t i = 0; i < possibleStops.size(); i++)
//    {
//      printf("%3zu - %s\n", i, possibleStops[i].c_str());
//    }
//    int destinationStopNr;
//    std::cin >> destinationStopNr;
//    string destinationStopName = possibleStops[destinationStopNr];
//
//    string input;
//    printf("\nPlease enter the hour of departure\n");
//    std::cin >> input;
//    char hour = atoi(input.c_str());
//
//    if (hour < 0 || hour > 23)
//      printf("Please enter an hour between 0 and 23\n");
//
//    printf("\nPlease enter the minute of departure\n");
//    std::cin >> input;
//    char minute = atoi(input.c_str());
//    if (minute < 0 || minute > 59)
//      printf("Please enter a minute between 0 and 59\n");
//    Time startTime(hour, minute, 0);
//    printf("computing path...\n");
//    routePlanner.computeShortestPath(departureStopName,
//                                     startTime,
//                                     destinationStopName
//                                    ,true);
//    printf("Path: \n");
//    for (size_t i = 0; i < routePlanner.optimalPaths.size(); i++)
//    {
//      printf("%s\n", routePlanner.optimalPaths[i].toString().c_str());
//    }
//    printf("Enter q to quit, other key to continue\n");
//    std::cin >> input;
//    if (input == "q" || input == "Q")
//      return 0;
//  }
  routePlanner.computeShortestPath(3, TRANSITNETWORK,
                                     Time(21,55,26),
                                     2,
                                     TRANSITNETWORK,
                                    true);
  for (size_t i = 0; i < routePlanner.optimalPaths.size(); i++)
  {
      printf("%s\n", routePlanner.optimalPaths[i].toString().c_str());
  }
//  WeekDate startTime(0, 23, 59);
//    printf("computing path...\n");
//    routePlanner.computeShortestPath(0, startTime,
//                                     4,true);
//    printf("Path: \n");
//    for (size_t i = 0; i < routePlanner.transitShortestPath.size(); i++)
//    {
//      std::cout << routePlanner.transitShortestPath[i] << std::endl;
//    }
  return 0;
}