#include <utility>
#include <algorithm>
#include <limits>
#include <vector>
#include <sstream>
#include <iostream>
#include <queue>
#include <string>
#include <functional>
#include <float.h>
#include <algorithm>
#include "./MulticriteriaDependentDijkstra.h"

using std::pair;
using std::cout;
using std::endl;
// _____________________________________________________________________________
// CLASS: TransitDependentDijkstra
// Dijkstra algorithm for transitnetwork based on dependent graph
MulticriteriaDependentDijkstra::MulticriteriaDependentDijkstra(
                                TransitNetworkDependent* roadNetwork)
{
  this->_tranGraph = roadNetwork;
  this->transferMax = std::numeric_limits<int>::max();
  this->timeMax = std::numeric_limits<int>::max();
}
// _____________________________________________________________________________
double MulticriteriaDependentDijkstra::computeShortestPath(string startStop, Time time,
                                      string endStop, bool buildPath)

{
  return computeShortestPath(_tranGraph->getNodeIdByTheStopName(startStop),
                             time,
                             _tranGraph->getNodeIdByTheStopName(endStop),
                             buildPath);
}

// _____________________________________________________________________________
double MulticriteriaDependentDijkstra::computeShortestPath(int startId,
                                            Time startTime,
                                            int endId, bool buildPath)
{
  std::priority_queue <pair<int, int>,
                  std::vector<pair<int, int> >,
                  std::greater<pair<int, int > > > _openNodeQueue;
  optimalPaths.clear();

  if (startId == -1)
  {
    printf("start stop is not known\n");
    return -1;
  }
  if (endId == -1)
  {
    printf("end stop is not known\n");
    return -1;
  }
  int transferTime = 4 * 60;
  _endStopFound = false;
  exploredNodes = 0;
  size_t gepopt = 0;

  labelSets.clear();
  labelSets.resize(_tranGraph->getNumNodes(), LabelSet(&labels, 4 * 60));
  labels.clear();
  int currentId = startId;
  int currentWight;
  int currentLabelId;
  Connection currentStopTime(startTime, startTime, -1);
  vector <int> currentCriterias;
  currentCriterias.push_back(0);
  currentCriterias.push_back(0);
  bool flag = labelSets[currentId].AddLabel(currentStopTime, currentCriterias,
                                   startId, 0, UNKNOWN, false, false, true, true);
  assert(flag);
  vector<EdgeDependent>* edges;
  int newId;
  //_openNodeQueue.push(std::pair<double, int> (0, labels.size()-1));
//  cout << "transferMax=" << transferMax << " timeMax=" << timeMax << endl;
  edges = _tranGraph->getOutEdges(startId);
  for (size_t i = 0; i < edges->size(); i++)
  {
    // get all new distances
    EdgeDependent& newEdge = (*edges)[i];
    vector <Connection> startConections;
    newId = newEdge.otherNodeId;
    newEdge.GetNextConectionOfAllTrips(currentStopTime, &startConections);
    for (size_t j = 0; j < startConections.size(); j++)
    {
      vector <int> newCriterias = currentCriterias;
      if (startConections[j].departureTime < currentStopTime.arrivalTime
              && currentStopTime.arrivalTime <= startConections[j].arrivalTime)
        newCriterias[0] = startConections[j]
               .arrivalTime.minusInSecondsDependent(currentStopTime.arrivalTime)
                        + 24 * 60 * 60;
      else
        newCriterias[0] = startConections[j].arrivalTime
                          .minusInSecondsDependent(currentStopTime.arrivalTime);
      newCriterias[1] = 0;
      {
//            startConections[j].startTime.printTime();
//            cout << "tripId="<<startConections[j].lineId << " newCr[0]=" << newCriterias[0] << endl;
//      cout << newId << " "<<endId << " weig="<< newCriterias[0]<<   endl;
      if (labelSets[newId].AddLabel(startConections[j], newCriterias,
              newId, 0, UNKNOWN, false, newId == endId, true, true))
      {
        _openNodeQueue.push(std::pair<int, int> (newCriterias[0],
                                                    labels.size() - 1));
      }
      }
    }
  }

  bool dijkstraFinished = _openNodeQueue.empty();
  while (!dijkstraFinished)
  {
    // get minimum of node queue
    currentLabelId = _openNodeQueue.top().second;
    currentId = labels[currentLabelId].NodeId;
    currentStopTime = labels[currentLabelId].connection;
    currentWight = _openNodeQueue.top().first;
//    labels[currentLabelId].connection.departureTime.printTime();
//    labels[currentLabelId].connection.arrivalTime.printTime();
//    cout << "tripId="<< labels[currentLabelId].connection.lineId <<" "<< labels[currentLabelId].criterias[1]<< " " << currentId << " " << currentWight<<endl;
    _openNodeQueue.pop();
    gepopt++;
    // check if already explored
    if (labels[currentLabelId].removed == true)
    {
       dijkstraFinished = _openNodeQueue.empty();
      // cout << "1" << endl;
       continue;
    }
    if (currentId == endId)
    {
       dijkstraFinished = _openNodeQueue.empty();
       continue;
    }
    if (labelSets[endId]._expandedFrom.size() != 0)
    {
      if (!labelSets[endId].labelMustBeAdded(
              labels[currentLabelId].criterias[0],
              labels[currentLabelId].criterias[1],
              labels[currentLabelId].connection.lineId, 0, false, true, true))
      {
        //labels[currentLabelId].removed = true;
        dijkstraFinished = _openNodeQueue.empty();
      }
    }
    // get outgoing edges from this node
    edges = _tranGraph->getOutEdges(currentId);
    // compute all new distances in loop
    for (size_t k = 0; k < edges->size(); k++)
      if (((*edges)[k].otherNodeId != labels[labels[currentLabelId].expandedFromLabelId].NodeId)
            && (*edges)[k].otherNodeId != currentId)
    {
      // get all new distances
      EdgeDependent& newEdge = (*edges)[k];
      newId = newEdge.otherNodeId;
      vector <Connection> newStopTimes;
      bool domination = true;
      newEdge.GetWight(currentStopTime, transferTime,
                       &newStopTimes, currentId == startId, true, &domination);
      if (newStopTimes.size() == 0)
        continue;
//      if ( newId == endId)
//        cout << currentId <<"-"<< newId << "*** " << newStopTimes.size()<<  " "
//             << currentLabelId
//             << " time=" << labels[currentLabelId].criterias[0] / 60
//             << " transfer=" << labels[currentLabelId].criterias[1]
//             << " line=" << labels[currentLabelId].connection.lineId << endl;
      for (size_t i = 0; i < newStopTimes.size(); i++)
      {
//        cout << "---" << endl;
//        newStopTimes[i].startTime.printTime();
//        newStopTimes[i].endTime.printTime();
        vector <int> newCriterias = labels[currentLabelId].criterias;
        if ((currentStopTime.lineId != newStopTimes[i].lineId &&
                 (currentStopTime.arrivalTime.secondsSinceDay() + 4 * 60 >
                  newStopTimes[i].departureTime.secondsSinceDay()
                && currentStopTime.arrivalTime.secondsSinceDay() <=
                  newStopTimes[i].arrivalTime.secondsSinceDay()))
           ||(currentStopTime.lineId == newStopTimes[i].lineId &&
                 (currentStopTime.arrivalTime.secondsSinceDay() >
                  newStopTimes[i].departureTime.secondsSinceDay()
                && currentStopTime.arrivalTime.secondsSinceDay() <=
                  newStopTimes[i].arrivalTime.secondsSinceDay())))
        {
          newCriterias[0] += 24 * 60 * 60 + newStopTimes[i]
                  .arrivalTime.minusInSecondsDependent(currentStopTime.arrivalTime);
        }
        else
        {
          newCriterias[0] += newStopTimes[i]
                  .arrivalTime.minusInSecondsDependent(currentStopTime.arrivalTime);
        }
        if (newStopTimes[i].lineId != currentStopTime.lineId)
          newCriterias[1] += 1;
        if (((newCriterias[0] < timeMax)||(newCriterias[1] < transferMax)
              || (newCriterias[0]==timeMax && newCriterias[1]== transferMax))
            && !((labelSets[endId]._expandedFrom.size() != 0)
                &&(!labelSets[endId].labelMustBeAdded(
              newCriterias[0],
              newCriterias[1],
              newStopTimes[i].lineId, 0, false, true, true)) ))
        if (labelSets[newId].AddLabel(newStopTimes[i], newCriterias,
                                      newId, currentLabelId, UNKNOWN, false, newId == endId, true, true))
        {
          _openNodeQueue.push(std::pair<int, int> (newCriterias[0] + newCriterias[1]*100000, labels.size()-1));
        }
      }
    }
    exploredNodes++;
    // for the rest of explorations to 4 min
    dijkstraFinished = _openNodeQueue.empty();
  }


  //cout << labelSets[endId]._expandedFrom.size() << endl;
  printf("exlored: %zu\n", exploredNodes);
  printf("poped  : %zu\n", gepopt);
  if (labelSets[endId]._expandedFrom.size() == 0)
  {
    //printf("shortest Path not found from %d to %d\n", startId, endId);
     if (buildPath)
       transitShortestPath.clear();
    return -1;
  }
  if (buildPath)
    reconstructShortestPath(startTime, startId, endId);
  // TODO(Robin): make it return actual length...
  return 0;
}

// _____________________________________________________________________________
void MulticriteriaDependentDijkstra::reconstructShortestPath(
                                                          const Time& startTime,
                                                          size_t startId,
                                                          size_t endId)
{
  transitShortestPath.clear();
  for (size_t i = 0; i < labelSets[endId]._expandedFrom.size(); i++)
    for (size_t j = 0; j < labelSets[endId]._expandedFrom[i].size(); j++)
      if (!labels[labelSets[endId]._expandedFrom[i][j]].removed)
      {
        //printf("i=%zu of %zu | j=%zu of %zu ", i,labelSets[endId]._expandedFrom.size(),j,labelSets[endId]._expandedFrom[j].size());
        size_t currentId = endId;
        size_t currentLabelId = (int)labelSets[endId]._expandedFrom[i][j];
        size_t oldLabelId = -1;
        size_t oldId;
        bool flag = true;

        string line;
        TransitPath newPath(startTime);
        std::stringstream buf (std::stringstream::in | std::stringstream::out);
        buf << "*********** PATH: " << i+1
            << " *********** TIME: " << (labels[currentLabelId].criterias[0]/60)
            << " *********** TRAN: " << labels[currentLabelId].criterias[1];
        line = buf.str();
        transitShortestPath.push_back(line);
        while (flag)
        {
          std::stringstream ss (std::stringstream::in | std::stringstream::out);
          int nodeId = labels[currentLabelId].NodeId;
          double latitude = _tranGraph->getNode(nodeId)->latitude;
          double longitude = _tranGraph->getNode(nodeId)->longitude;
          Time arrivalTime = labels[currentLabelId].connection.arrivalTime;
          Time departureTime;  // will be left as dummy time in case
                               // we are at last node
          int arrivedWithTripId = labels[currentLabelId].connection.lineId;
          if(currentId != endId)
            departureTime = labels[oldLabelId].connection.departureTime;
          newPath.addStation(TransitPath::TRANSITNODE, nodeId, latitude, longitude,
                             arrivalTime, departureTime,
                             arrivedWithTripId);

          if (currentId == endId)
          {
            // no departure time at end id
            ss << _tranGraph->getStopName(currentId)
               << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
               << "\t\t\tTripId:" << labels[currentLabelId].connection.lineId;
          }
          else if (currentId != startId)
          {
            ss << _tranGraph->getStopName(currentId)
               << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
               << "\tDEP:" << (int)labels[oldLabelId].connection.departureTime.hour()
               << ":" << (int)labels[oldLabelId].connection.departureTime.minute()
               << ":" << (int)labels[oldLabelId].connection.departureTime.second()
               << "\tTripId:" << labels[currentLabelId].connection.lineId;
          }
          else
          {
            ss << _tranGraph->getStopName(currentId)
               << "\tARR:" << (int)labels[currentLabelId].connection.arrivalTime.hour()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.minute()
               << ":" << (int)labels[currentLabelId].connection.arrivalTime.second()
               << "\tDEP:" << (int)labels[oldLabelId].connection.departureTime.hour()
               << ":" << (int)labels[oldLabelId].connection.departureTime.minute()
               << ":" << (int)labels[oldLabelId].connection.departureTime.second();
          }
          oldId = currentId;
          oldLabelId = currentLabelId;
          flag = currentId != startId;
          if (flag)
          {

            currentLabelId = labels[currentLabelId].expandedFromLabelId;
            currentId = labels[currentLabelId].NodeId;
          }
          line = ss.str();
          transitShortestPath.push_back(line);
        }
        newPath.reverse();
        optimalPaths.push_back(newPath);
        transitShortestPath.push_back("*****************************************");
       // printf("Ok!\n");
      }
}
