/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

#ifndef ROUTEPLANNINGV2_NODE_H_
#define ROUTEPLANNINGV2_NODE_H_
#include <string>
#include "./Time.h"
using std::string;

enum NodeType
{
  ARRIVALNODE, TRANSFERNODE, DEPARTURENODE
};

class NodeExpanded
{
 public:
  NodeExpanded(int id, double latitude, double longitude);
  NodeExpanded(int id, double latitude, double longitude, int stopId, Time time,
       NodeType type);
  int id;
  double latitude;
  double longitude;
  NodeType type;
  // the stop to which this node belongs
  int stopId;
  Time time;
  // calculate the straight line distance to another node
  // Now we multiply by 111 as this should be correct in theory...
  // WARNING: since we don't know how to do this properly we simply
  // take coordiantes as if they were on a plane and multiply result by
  // 1000000 :)
  double straightLineDistanceTo(const NodeExpanded& otherNode);
};

// comparing the nodes by time first
// then by type as a tie breaker, if time and type are both equal
// use node id as tie breaker
// this operator defines a weak ordering
struct NodePointerTimeAndTypeComparator
{
  bool operator() (const NodeExpanded* nodeA, const NodeExpanded* nodeB);
};


// comparing the nodes by time first
// then by type as a tie breaker, if time and type are both equal
// use node id as tie breaker
// this operator defines a weak ordering
struct NodeTimeAndTypeComparator
{
  bool operator() (const NodeExpanded& nodeA, const NodeExpanded& nodeB);
};
#endif  // ROUTEPLANNINGV2_NODE_H_

