/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning Summer Semester 2011
 * Uni Freiburg
 */

// Time class with hours and minutes
// hours are from 0 to 23, minutes from 0 to59...
#ifndef ROUTEPLANNINGV2_TIME_H_
#define ROUTEPLANNINGV2_TIME_H_
#include <string>
#include <iostream>
using std::string;
using std::ostream;
class Time
{
 public:
  Time();
  Time(char hour, char minutes, char seconds);
  Time (const Time& originalTime);
  explicit Time(int secondsSinceDay);
  // construct the time out of a string like "25:30:00" for 1:30 am the next day
  // or 16:30:45 for 16:30 and 45 seconds on this day..
  explicit Time(string GTFSTimeString);

  bool operator< (const Time& otherTime) const;
  bool operator<= (const Time& otherTime) const;
  bool operator== (const Time& otherTime) const;

  int minusInSeconds(const Time & otherTime) const;
  // always gives positive value, assumes this time is one day after
  // the othertime, if it would be earlier on the same day, e.g.:
  // Time a (0, 1, 0);
  // Time b (23, 59, 30);
  // a.minusInSecondsDependent(b) => 90 (seconds).
  // normal minus in minutes wouldve given 90 - 24 * 3600 = -86 310
  int minusInSecondsDependent(const Time & otherTime) const;

  // get day name in english
  //string getDayName();
  void setHour(char hour);
  void setMinute(char minute);
  void setSecond(char second);
  void setSecondsSinceDay(int secondsSinceDay);
  void addSeconds(int seconds);
  int hour() const;
  // minutes in the hour
  int minute() const;
  // seconds in the minute
  int second() const;
  // get minutes since start of day (sicne 0:00 AM)
  int secondsSinceDay() const;
  string toString() const;
  void printTime() const;
  string createJSON() const;
 private:

  // seconds since the start of the day...
  int _secondsSinceDay;
};

ostream& operator<< (ostream& outStream, const Time& time);

#endif  // ROUTEPLANNINGV2_TIME_H_
