
#include "./TransferPatterns.h"
#include <utility>
#include <unordered_map>
#include <unordered_set>
#include <algorithm>
#include <vector>
#include <queue>
#include <string>
#include <functional>
#include "./TransitNetworkExpanded.h"
#include "./Dijkstra.h"
using std::endl;
using std::cout;
// _____________________________________________________________________________
TransferPatterns::TransferPatterns(TransitNetworkExpanded* roadNetwork,
                                   DistanceHeuristic* heuristic)
                                  : SimpleDijkstra(roadNetwork, heuristic)
{
  _tranGraph = roadNetwork;
  transferPatterns = new vector< vector<int> >;
  transferPatterns->resize(_tranGraph->getNumStops());
  for (size_t i = 0; i< transferPatterns->size(); i++)
    transferPatterns->at(i).resize(_tranGraph->getNumStops());
  this->heuristic = heuristic;
}
// _____________________________________________________________________________
// Parse int number to string
//
string intToString(int64_t num)
{
  std::stringstream sstr;
  sstr << num;
  string buf;
  sstr >> buf;
  return buf;
}
// _____________________________________________________________________________
void TransferPatterns::doPrecomputation()
{
  for (size_t i = 0; i < transferPatterns->size(); i++)
  {
    cout << "Stop : " << i << endl;
    resetAll();
    vector <int> startNodes;
    _tranGraph->getStopNodeIds(_tranGraph->getStopName(i), &startNodes);
    for (size_t j = 0; j < startNodes.size()-1; j++)
      _openNodeQueue.push(pair<double, int>(0, startNodes[j]));
    computeShortestPath(startNodes[startNodes.size()-1]);
    for (size_t j = 0; j < transferPatterns->size(); j++)
    {
      if (i == j) continue;
      vector <int> stopNodes;
      _tranGraph->getStopNodeIds(_tranGraph->getStopName(j),
                                           &stopNodes);
      vector <int> optimizedStopNodes;
      size_t nextNode = 0;
      while ((nextNode < stopNodes.size())
             &&(timesFromStart[stopNodes[nextNode]] == 0)) nextNode++;
      if (nextNode == stopNodes.size()) continue;
      optimizedStopNodes.push_back(stopNodes[nextNode]);
      size_t k = nextNode;
      //cout << "stoptnodesSize "<<stopNodes->size() << endl;
      int test=0;
      while (k < stopNodes.size() - 1)
      {
        k++;
        if (timesFromStart[stopNodes[k]]==0) continue;
        NodeExpanded* node1 = _tranGraph->getNode(stopNodes[nextNode]);
        NodeExpanded* node2 = _tranGraph->getNode(stopNodes[k]);
         //cout << "-" << endl;
        //cout << "node2->time.minusInMinutes(node1->time)="  << node2->time.minusInMinutesHACK(node1->time) << endl;
        //cout << "timesFromStart[stopNodes->at(nextNode)]"  << timesFromStart[stopNodes->at(nextNode)] << endl;
        //cout << "timesFromStart[stopNodes->at(k)]"  << timesFromStart[stopNodes->at(k)] << endl;
        if ((timesFromStart[stopNodes[nextNode]]
           + node2->time.minusInSeconds(node1->time))
            > timesFromStart[stopNodes[k]])
        {
          optimizedStopNodes.push_back(stopNodes[k]);
          nextNode = k;
          test++;
        }
        
      }
      //cout << "test"  << test << endl;
      std::unordered_set<string> patterns;
      size_t buf=j;
      for (size_t wayId = 0; wayId < optimizedStopNodes.size(); wayId++)
      {
        //cout << "-" << endl;
        string pattern = intToString(j) + ".";
        size_t nextId = expandedFrom[optimizedStopNodes[wayId]].first;
        while (static_cast<size_t>(_tranGraph->getNode(nextId)->stopId) != i)
        {
          //cout << "stopId:" << _tranGraph->getNode(nextId)->stopId << endl;
          if ((_tranGraph->getNode(nextId)->type == TRANSFERNODE)
           &&(static_cast<size_t>(_tranGraph->getNode(nextId)->stopId) != buf))
          {
            pattern += intToString(_tranGraph->getNode(nextId)->stopId) + ".";
            buf = _tranGraph->getNode(nextId)->stopId;
          }
          nextId = expandedFrom[nextId].first;
        }
        pattern += intToString(i);
        //cout << pattern << endl;
        patterns.insert(pattern);
      }
      transferPatterns->at(i)[j] = patterns.size();
     // cout << "transferPatterns["<< i << "," << j << "]=" << transferPatterns->at(i)[j] << endl;
    }
  }
  vector<size_t> patternsFreq;
  patternsFreq.resize(7);
  pair<int,int> maxStopPair;
  int maxPatNum=-1;
  int nodeNum = 0;
  for (size_t i = 0; i < transferPatterns->size(); i++)
    for (size_t j = 0; j < transferPatterns->size(); j++)
      if (i != j)
      {
       // cout << "transferPatterns["<< i << "," << j << "]=" << transferPatterns->at(i)[j] << endl;
        if ((transferPatterns->at(i)[j]) > maxPatNum)
        {
          maxPatNum = transferPatterns->at(i)[j];
          maxStopPair.first = i;
          maxStopPair.second = j;
        }
        if (transferPatterns->at(i)[j] >= 100)
        {
          patternsFreq[6]++;
          nodeNum++;
        } else if (transferPatterns->at(i)[j] >= 50)
        {
          patternsFreq[5]++;
          nodeNum++;
        } else if (transferPatterns->at(i)[j] >= 20)
        {
          patternsFreq[4]++;
          nodeNum++;
        } else if (transferPatterns->at(i)[j] >= 10)
        {
          patternsFreq[3]++;
          nodeNum++;
        } else if (transferPatterns->at(i)[j] >= 5)
        {
          patternsFreq[2]++;
          nodeNum++;
        } else if (transferPatterns->at(i)[j] >= 1)
        {
          patternsFreq[1]++;
          nodeNum++;
        }
        else
        {
          patternsFreq[0]++;
          nodeNum++;
        }
      }
  cout << "___________________________" <<  endl;
  cout << "0     : " << static_cast<double>(patternsFreq[0])/nodeNum << endl;
  cout << "1-4   : " << static_cast<double>(patternsFreq[1])/nodeNum << endl;
  cout << "5-9   : " << static_cast<double>(patternsFreq[2])/nodeNum << endl;
  cout << "10-19 : " << static_cast<double>(patternsFreq[3])/nodeNum << endl;
  cout << "20-49 : " << static_cast<double>(patternsFreq[4])/nodeNum << endl;
  cout << "50-99 : " << static_cast<double>(patternsFreq[5])/nodeNum << endl;
  cout << ">=100 : " << static_cast<double>(patternsFreq[6])/nodeNum << endl;
  cout << "Station pair with the bigest number of transfer patterns : "
       << _tranGraph->getStopName(maxStopPair.first) << " (with id "
       << maxStopPair.first << ") -> "
       << _tranGraph->getStopName(maxStopPair.second) << " (with id "
       << maxStopPair.second << ")" << endl;
  cout << "It has " << maxPatNum << " transfer patterns" << endl;

}
// _____________________________________________________________________________
void TransferPatterns::computeShortestPath(int startId)
{
  bool checkNodes = false;
  bool checkEdges = false;
  bool rememberEdges = true;
  bool forward = true;
  Dijkstra::computeShortestPathToAll(startId, checkNodes,
                                                     checkEdges,
                                                     rememberEdges, forward);
}
// _____________________________________________________________________________
void TransferPatterns::resetNode(int nodeId)
{

}