/* Copyright Robin Schirrmeister and Simon Skilevic
 * Lecture Efficient Route Planning summer semester 2011
 * Uni Freiburg
 * Testclass for Transit network
 */

#include <stdio.h>
#include <gtest/gtest.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unordered_map>
#include <stdlib.h>
#include <string>
#include <utility>
#include <vector>
#include "./CsvParser.h"
#include "./TransitNetworkExpanded.h"
#include "./Dijkstra.h"
#include "./ExpandedDijkstra.h"
#include "./TransitParser.h"

using std::string;
using std::unordered_map;


TEST(TransitNetworkTest, parseCalendarFile)
{
  TransitNetworkExpanded transitNetwork;
  string calendarFileName = "calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);
  // try out several testDays:
  char testDay = 0;
  unordered_set<string> validServiceIds;
  transitNetwork.parseCalendarFile(calendarFileName, testDay, &validServiceIds);
  // on monday weekday and fullweek should be true, weekend not..
  ASSERT_EQ(0, validServiceIds.count("WE"));
  ASSERT_EQ(1, validServiceIds.count("WD"));
  ASSERT_EQ(1, validServiceIds.count("FW"));

  // same for thursday
  testDay = 3;
  validServiceIds.clear();
  transitNetwork.parseCalendarFile(calendarFileName, testDay, &validServiceIds);
  // on monday weekday and fullweek should be true, weekend not..
  ASSERT_EQ(0, validServiceIds.count("WE"));
  ASSERT_EQ(1, validServiceIds.count("WD"));
  ASSERT_EQ(1, validServiceIds.count("FW"));

  // now test saturday and sunday, weekend should be true, weekday not...;
  testDay = 5;
  validServiceIds.clear();
  transitNetwork.parseCalendarFile(calendarFileName, testDay, &validServiceIds);
  ASSERT_EQ(1, validServiceIds.count("WE"));
  ASSERT_EQ(0, validServiceIds.count("WD"));
  ASSERT_EQ(1, validServiceIds.count("FW"));

  testDay = 6;
  validServiceIds.clear();
  transitNetwork.parseCalendarFile(calendarFileName, testDay, &validServiceIds);
  ASSERT_EQ(1, validServiceIds.count("WE"));
  ASSERT_EQ(0, validServiceIds.count("WD"));
  ASSERT_EQ(1, validServiceIds.count("FW"));
  remove(calendarFileName.c_str());
}


TEST(TransitNetworkTest, parseTripsFile)
{
  // first create both calendar file and trips file, then parse them
  TransitNetworkExpanded transitNetwork;
  string calendarFileName = "calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,thursday,"
                        "friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);
  string tripFileName = "trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n");
  fclose(tripFile);
  // test for monday to friday, only weekday and full week trips
  // should be true...
  unordered_set<string> validServiceIds;
  unordered_map<string, int> tripIdNameToTripId;
  for (char testDay = 0; testDay < 5; testDay++)
  {
    validServiceIds.clear();
    tripIdNameToTripId.clear();
    transitNetwork.parseCalendarFile(calendarFileName, testDay,
                                     &validServiceIds);
    transitNetwork.parseTripsFile(tripFileName, validServiceIds,
                                  &tripIdNameToTripId);
    ASSERT_EQ(0, tripIdNameToTripId.count("U7WE"));
    ASSERT_EQ(1, tripIdNameToTripId.count("U7WD"));
    ASSERT_EQ(1, tripIdNameToTripId.count("U7FW"));
    ASSERT_EQ(1, tripIdNameToTripId.count("U9FW"));
  }
  // test for saturday and sunday, only weekend and full week trips
  // should be true
  for (char testDay = 5; testDay < 7; testDay++)
  {
    validServiceIds.clear();
    tripIdNameToTripId.clear();
    transitNetwork.parseCalendarFile(calendarFileName, testDay,
                                     &validServiceIds);
    transitNetwork.parseTripsFile(tripFileName, validServiceIds,
                                  &tripIdNameToTripId);
    ASSERT_EQ(1, tripIdNameToTripId.count("U7WE"));
    ASSERT_EQ(0, tripIdNameToTripId.count("U7WD"));
    ASSERT_EQ(1, tripIdNameToTripId.count("U7FW"));
    ASSERT_EQ(1, tripIdNameToTripId.count("U9FW"));
  }
  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
}

TEST(TransitNetworkTest, parseStopsFile)
{
    // create stop file
  TransitNetworkExpanded transitNetwork;
  string stopsFileName = "stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,stop_url,"
    "location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U72,Berliner Strasse, Zu U9 :),52.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U91,Berliner Strasse,umsteigen,37.75713,-122.418982,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);
  unordered_map<string, int > stopIdNameToId;
  vector<Stop> stops;
  transitNetwork.parseStopsFile(stopsFileName, &stops, &stopIdNameToId);
  remove(stopsFileName.c_str());
  // first check that the id to name vector in the network is correct
  ASSERT_EQ(6, transitNetwork._stops.size());
  ASSERT_EQ("Eisenacher Strasse", transitNetwork._stops.at(0).stopName);
  ASSERT_EQ("Bayerischer Platz", transitNetwork._stops.at(1).stopName);
  ASSERT_EQ("Berliner Strasse", transitNetwork._stops.at(2).stopName);
  ASSERT_EQ("Blissestrasse", transitNetwork._stops.at(3).stopName);
  // we force stop names to be unique..
  ASSERT_EQ("Berliner Strasse U91", transitNetwork._stops.at(4).stopName);
  ASSERT_EQ("Guentzelstrasse", transitNetwork._stops.at(5).stopName);
  // now check the map... first all ids then some coordinates
  ASSERT_EQ(6, stopIdNameToId.size());
  ASSERT_EQ(0, stopIdNameToId["U70"]);
  ASSERT_EQ(1, stopIdNameToId["U71"]);
  ASSERT_EQ(2, stopIdNameToId["U72"]);
  ASSERT_EQ(3, stopIdNameToId["U73"]);
  ASSERT_EQ(4, stopIdNameToId["U91"]);
  ASSERT_EQ(5, stopIdNameToId["U92"]);


  ASSERT_EQ(37.728631,
          transitNetwork._stops.at(stopIdNameToId["U70"]).latitude);
  ASSERT_EQ(-122.431282,
            transitNetwork._stops.at(stopIdNameToId["U70"]).longitude);
  ASSERT_EQ(37.728631,
            transitNetwork._stops.at(stopIdNameToId["U71"]).latitude);
  ASSERT_EQ(-122.433282,
            transitNetwork._stops.at(stopIdNameToId["U71"]).longitude);
  ASSERT_EQ(37.75713,
          transitNetwork._stops.at(stopIdNameToId["U91"]).latitude);
  ASSERT_EQ(-122.418982,
            transitNetwork._stops.at(stopIdNameToId["U91"]).longitude);
}


TEST(TransitNetworkTest, parseFrequenciesFile)
{
    // create frequencies file, times will overlap for the trips but so what
  TransitNetworkExpanded transitNetwork;
  string frequenciesFileName = "frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:20,25:30:00,900\n"
    "U7WE,00:00:00,24:00:15,600\n"
    "U7WD,05:30:32,26:30:17,300\n"
    "U7FW,05:30:00,25:00:00,1200\n"
    "U9FW,2:30:18,5:00:22,1200\n");
  fclose(frequenciesFile);
  unordered_map<string, vector<TransitParser::TripFrequency> >
      tripToFrequencyAndTimes;
  transitNetwork.parseFrequenciesFile(frequenciesFileName,
                                      &tripToFrequencyAndTimes);
  remove(frequenciesFileName.c_str());
  ASSERT_EQ(4, tripToFrequencyAndTimes.size());
  ASSERT_EQ(2, tripToFrequencyAndTimes["U9FW"].size());
  ASSERT_EQ(900, tripToFrequencyAndTimes["U9FW"][0].intervalInSeconds);
  ASSERT_EQ(5, tripToFrequencyAndTimes["U9FW"][0].startHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U9FW"][0].startMinute);
  ASSERT_EQ(20, tripToFrequencyAndTimes["U9FW"][0].startSecond);
  ASSERT_EQ(25, tripToFrequencyAndTimes["U9FW"][0].endHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U9FW"][0].endMinute);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U9FW"][0].endSecond);
  ASSERT_EQ(1200, tripToFrequencyAndTimes["U9FW"][1].intervalInSeconds);
  ASSERT_EQ(2, tripToFrequencyAndTimes["U9FW"][1].startHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U9FW"][1].startMinute);
  ASSERT_EQ(18, tripToFrequencyAndTimes["U9FW"][1].startSecond);
  ASSERT_EQ(5, tripToFrequencyAndTimes["U9FW"][1].endHour);
  ASSERT_EQ(00, tripToFrequencyAndTimes["U9FW"][1].endMinute);
  ASSERT_EQ(22, tripToFrequencyAndTimes["U9FW"][1].endSecond);
  ASSERT_EQ(1, tripToFrequencyAndTimes["U7WE"].size());
  ASSERT_EQ(600, tripToFrequencyAndTimes["U7WE"][0].intervalInSeconds);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7WE"][0].startHour);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7WE"][0].startMinute);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7WE"][0].startSecond);
  ASSERT_EQ(24, tripToFrequencyAndTimes["U7WE"][0].endHour);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7WE"][0].endMinute);
  ASSERT_EQ(15, tripToFrequencyAndTimes["U7WE"][0].endSecond);
  ASSERT_EQ(1, tripToFrequencyAndTimes["U7WD"].size());
  ASSERT_EQ(300, tripToFrequencyAndTimes["U7WD"][0].intervalInSeconds);
  ASSERT_EQ(5, tripToFrequencyAndTimes["U7WD"][0].startHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U7WD"][0].startMinute);
  ASSERT_EQ(32, tripToFrequencyAndTimes["U7WD"][0].startSecond);
  ASSERT_EQ(26, tripToFrequencyAndTimes["U7WD"][0].endHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U7WD"][0].endMinute);
  ASSERT_EQ(17, tripToFrequencyAndTimes["U7WD"][0].endSecond);
  ASSERT_EQ(1, tripToFrequencyAndTimes["U7FW"].size());
  ASSERT_EQ(1200, tripToFrequencyAndTimes["U7FW"][0].intervalInSeconds);
  ASSERT_EQ(5, tripToFrequencyAndTimes["U7FW"][0].startHour);
  ASSERT_EQ(30, tripToFrequencyAndTimes["U7FW"][0].startMinute);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7FW"][0].startSecond);
  ASSERT_EQ(25, tripToFrequencyAndTimes["U7FW"][0].endHour);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7FW"][0].endMinute);
  ASSERT_EQ(0, tripToFrequencyAndTimes["U7FW"][0].endSecond);
}


TEST(TransitNetworkTest, addTrip)
{
  // small trip over 3 stops
  TransitNetworkExpanded transitNetwork;
  TransitParser::TripNode node1("10:9:13", 0, ARRIVALNODE);
  TransitParser::TripNode node2("10:11:25", 0, TRANSFERNODE);
  TransitParser::TripNode node3("10:9:43", 0, DEPARTURENODE);
  TransitParser::TripNode node4("10:12:13", 1, ARRIVALNODE);
  TransitParser::TripNode node5("10:14:25", 1, TRANSFERNODE);
  TransitParser::TripNode node6("10:13:35", 1, DEPARTURENODE);
  TransitParser::TripNode node7("10:14:45", 2, ARRIVALNODE);
  TransitParser::TripNode node8("10:15:9", 2, TRANSFERNODE);
  TransitParser::TripNode node9("10:16:6", 2, DEPARTURENODE);
  vector<TransitParser::TripNode> trip;
  trip.push_back(node1);
  trip.push_back(node2);
  trip.push_back(node3);
  trip.push_back(node4);
  trip.push_back(node5);
  trip.push_back(node6);
  trip.push_back(node7);
  trip.push_back(node8);
  trip.push_back(node9);
  // insert dummy stops
  transitNetwork._stops.push_back(Stop(0, "stop1", 3.3, 4.5));
  transitNetwork._stops.push_back(Stop(1, "stop1", 5.3, 2.5));
  transitNetwork._stops.push_back(Stop(2, "stop1", 6.3, 2.5));
  // trip goes from 10:10 am till 2:09 am the next day, only every hour
  // to make test faster..
  int frequencyInSeconds = 3600;
  TransitParser::TripFrequency frequency(10, 9, 13, 26, 10, 20,
                                          frequencyInSeconds);
  ASSERT_EQ(10, frequency.startHour);
  ASSERT_EQ(9, frequency.startMinute);
  ASSERT_EQ(13, frequency.startSecond);
  ASSERT_EQ(26, frequency.endHour);
  ASSERT_EQ(10, frequency.endMinute);
  ASSERT_EQ(20, frequency.endSecond);

  // 3 stop ids...
  vector<vector<int> > stopIdsToNodeIds;
  stopIdsToNodeIds.resize(3);
  int tripId = -1;
  printf("adding trip\n");
  transitNetwork.addTrip(trip, frequency,
                         &stopIdsToNodeIds, tripId);
  printf("done\n");
  // 9 nodes per trip, 17 trips per day
  ASSERT_EQ(9 * 17, transitNetwork.getNumNodes());
  ASSERT_EQ(3 * 17, stopIdsToNodeIds[0].size());
  ASSERT_EQ(3 * 17, stopIdsToNodeIds[1].size());
  ASSERT_EQ(3 * 17, stopIdsToNodeIds[2].size());

  // check all nodes assuming an order arrival, transfer departure while parsing
  for (size_t i = 0; i < transitNetwork.getNumNodes(); i++)
  {
    NodeExpanded* node = transitNetwork.getNode(i);
    ASSERT_EQ(i, node->id);
    // check coordinates
    if (i % 9 < 3)
    {
      ASSERT_EQ(3.3, node->latitude);
      ASSERT_EQ(4.5, node->longitude);
    }
    else if (i % 9 < 6)
    {
      ASSERT_EQ(5.3, node->latitude);
      ASSERT_EQ(2.5, node->longitude);
    }
    else
    {
      ASSERT_EQ(6.3, node->latitude);
      ASSERT_EQ(2.5, node->longitude);
    }
    // arrival node should have edges to transfer node and departure node
    if (i % 3 == 0)
    {
      ASSERT_EQ(node->type, ARRIVALNODE);
      vector<Edge>* outEdges = transitNetwork.getOutEdges(i);
      ASSERT_EQ(2, outEdges->size());
      NodeExpanded* transferNode = transitNetwork.getNode(outEdges->at(0)
                                                          .otherNodeId);
      ASSERT_EQ(node->stopId, transferNode->stopId);
      ASSERT_EQ(TRANSFERNODE, transferNode->type);
      NodeExpanded* departureNode = transitNetwork.getNode(outEdges->at(1)
                                                            .otherNodeId);
      ASSERT_EQ(node->stopId, departureNode->stopId);
      ASSERT_EQ(DEPARTURENODE, departureNode->type);
    }
    // transfer node should not have any edges right now
    if (i % 3 == 1)
    {
      ASSERT_EQ(node->type, TRANSFERNODE);
      vector<Edge>* outEdges = transitNetwork.getOutEdges(i);
      ASSERT_EQ(0, outEdges->size());
    }
    // departure node should have an edge to arrival node..
    if (i % 3 == 2)
    {
      ASSERT_EQ(node->type, DEPARTURENODE);
      vector<Edge>* outEdges = transitNetwork.getOutEdges(i);
      // last departure nodes should not have any edges
      if (i % 9 == 8)
      {
        ASSERT_EQ(0, outEdges->size());
      }
      else
      {
        ASSERT_EQ(1, outEdges->size());
        NodeExpanded* arrivalNode = transitNetwork.getNode(outEdges->at(0)
                                                           .otherNodeId);
        ASSERT_EQ(node->stopId + 1, arrivalNode->stopId);
        ASSERT_EQ(ARRIVALNODE, arrivalNode->type);
      }
    }
  }
  // some specific tests
  // check edge from first node to second node
  ASSERT_EQ(132, transitNetwork.getOutEdges(0)->at(0).time);
  // first to third
  ASSERT_EQ(30, transitNetwork.getOutEdges(0)->at(1).time);
  // third to fourth
  ASSERT_EQ(150, transitNetwork.getOutEdges(2)->at(0).time);
  // seventh to eighth
  ASSERT_EQ(24, transitNetwork.getOutEdges(6)->at(0).time);
  // seventh to ninth
  ASSERT_EQ(81, transitNetwork.getOutEdges(6)->at(1).time);
}

TEST(TransitNetworkTest, parseFromGTFSFiles)
{
  TransitNetworkExpanded transitNetwork;
  string testDirectory = "GTFSTempParseTest";
  int couldCreateDirectory = mkdir(testDirectory.c_str(), S_IRWXU | S_IRWXG |
                                                          S_IROTH | S_IXOTH);
  if (couldCreateDirectory != 0)
  {
    printf("couldn't create testDirectory: %s\n", testDirectory.c_str());
    // make test crash if it directory could not be created...
    ASSERT_TRUE(false);
  }
  string calendarFileName = testDirectory + "/calendar.txt";
  FILE* calendarFile = fopen(calendarFileName.c_str(), "w");
  fprintf(calendarFile, "service_id,monday,tuesday,wednesday,"
                        "thursday,friday,saturday,sunday,start_date,end_date\n"
                        "WE,0,0,0,0,0,1,1,20060701,20060731\n"
                        "WD,1,1,1,1,1,0,0,20060701,20060731\n"
                        "FW,1,1,1,1,1,1,1,20060701,20060731\n");
  fclose(calendarFile);

  string tripFileName = testDirectory + "/trips.txt";
  FILE* tripFile = fopen(tripFileName.c_str(), "w");
  fprintf(tripFile, "route_id,service_id,trip_id,trip_headsign,block_id\n"
                        "A,WE,U7WE,Downtown,1\n"
                        "A,WD,U7WD,Downtown,2\n"
                        "A,FW,U7FW,Downtown,2\n"
                        "A,FW,U9FW,Downtown,2\n");
  fclose(tripFile);

  string stopsFileName = testDirectory + "/stops.txt";
  FILE* stopsFile = fopen(stopsFileName.c_str(), "w");
  fprintf(stopsFile,
    "stop_id,stop_name,stop_desc,stop_lat,stop_lon,"
    "stop_url,location_type,parent_stop\n"
    "U70,Eisenacher Strasse,Pizza :),37.728631,-122.431282,,\n"
    "U71,Bayerischer Platz,Mein Zuhause :),37.728631,-122.433282,,\n"
    "U9_U7_1,Berliner Strasse, Zu U9 :),52.74103,-122.422482,,\n"
    "U73,Blissestrasse,was will man da?,37.75223,-122.418581,,\n"
    "U92,Guentzelstrasse,hmmm,37.761829,-122.419382,,\n");
  fclose(stopsFile);

  string frequenciesFileName = testDirectory + "/frequencies.txt";
  FILE* frequenciesFile = fopen(frequenciesFileName.c_str(), "w");
  fprintf(frequenciesFile,
    "trip_id,start_time,end_time,headway_secs\n"
    "U9FW,05:30:03,25:30:00,10800\n"
    "U7WE,00:00:00,24:00:00,5400\n"
    "U7WD,05:29:30,26:30:00,3600\n"
    "U7FW,05:30:00,25:30:00,7200\n");
  fclose(frequenciesFile);

  string stopTimesFileName = testDirectory + "/stop_times.txt";
  FILE* stopTimesFile = fopen(stopTimesFileName.c_str(), "w");
  fprintf(stopTimesFile,
    "trip_id,arrival_time,departure_time,stop_id,"
    "stop_sequence,pickup_type,dropoff_type\n"
    "U7FW,5:30:0,5:30:0,U70,1,0,0,0\n"
    "U7FW,5:32:0,5:33:0,U71,2,0,0,0\n"
    "U7FW,5:34:0,5:35:0,U9_U7_1,3,0,0,0\n"
    "U7FW,5:36:0,5:37:0,U73,4,0,0,0\n"
    "U7WE,0:0:0,0:1:0,U71,1,0,0,0\n"
    "U7WE,0:4:0,0:6:0,U9_U7_1,2,0,0,0\n"
    "U7WD,5:29:30,05:31:02,U70,1,0,0,0\n"
    "U7WD,5:32:0,05:33:20,U71,2,0,0,0\n"
    "U7WD,5:34:0,05:35:0,U9_U7_1,3,0,0,0\n"
    "U9FW,5:30:03,5:32:0,U9_U7_1,1,0,0,0\n"
    "U9FW,5:34:0,05:38:0,U92,2,0,0,0\n");
  // last stop transfer and departure time equal if waiting time = 4 minutes
  fclose(stopTimesFile);
  char testDay = 0;
  int transferTimeInSeconds = 4 * 60;
  transitNetwork.parseFromGTFSFiles(testDirectory, testDay,
                                    transferTimeInSeconds);

  // remove files and directory...
  remove(calendarFileName.c_str());
  remove(tripFileName.c_str());
  remove(stopsFileName.c_str());
  remove(frequenciesFileName.c_str());
  remove(stopTimesFileName.c_str());
  int couldRemoveDirectory = rmdir(testDirectory.c_str());
  if (couldRemoveDirectory != 0)
    printf("couldn't remove directory %s\n", testDirectory.c_str());

  // check that each trip available on monday exists by checking characteristic
  // edges
  vector<int> nodeIdsAtStop;
  transitNetwork.getStopNodeIds("Eisenacher Strasse", &nodeIdsAtStop);
  // ...check that it has edge to bayerischer platz
  // they are sorted by time, first node should be an arrivalnode at 0:29:30
  NodeExpanded* node;
  node = transitNetwork.getNode(nodeIdsAtStop[0]);
  ASSERT_EQ(ARRIVALNODE, node->type);
  ASSERT_EQ(0, node->time.hour());
  ASSERT_EQ(29, node->time.minute());
  ASSERT_EQ(30, node->time.second());
  // then comes the departure at 0:31:02
  node = transitNetwork.getNode(nodeIdsAtStop[1]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(0, node->time.hour());
  ASSERT_EQ(31, node->time.minute());
  ASSERT_EQ(02, node->time.second());
  // check that there is one edge edges to bayerischer platz
  vector<Edge>* edges = transitNetwork.getOutEdges(node->id);
  ASSERT_EQ(1, edges->size());
  ASSERT_EQ("Bayerischer Platz", transitNetwork.getStopName(
                             transitNetwork.getNode(edges->at(0).otherNodeId)->
                              stopId));
  // check some more nodes:
  // then transfer at 0:33:0
  node = transitNetwork.getNode(nodeIdsAtStop[2]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(Time(0, 33, 30), node->time);

  // same trip at 1:29:30 but also the u7 fullweek trip starting at 0:30:0
  node = transitNetwork.getNode(nodeIdsAtStop[3]);
  ASSERT_EQ(ARRIVALNODE, node->type);
  ASSERT_EQ(Time(1, 29, 30), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[4]);
  ASSERT_EQ(ARRIVALNODE, node->type);
  ASSERT_EQ(Time(1, 30, 0), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[5]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(Time(1, 30, 0), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[6]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(Time(1, 31, 2), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[7]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(Time(1, 33, 30), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[8]);
  ASSERT_EQ(TRANSFERNODE, node->type);
  ASSERT_EQ(Time(1, 34, 0), node->time);
  // now check that berlienr strasse has after the nodes from the U7 trips
  // the arrival node at 5:30:03
  // then de departure ndoe at 5:32:00 to guentzelstrasse
  // check whether nodes have correct stop ids...
  nodeIdsAtStop.clear();
  transitNetwork.getStopNodeIds("Berliner Strasse", &nodeIdsAtStop);
  node = transitNetwork.getNode(nodeIdsAtStop[12]);
  ASSERT_EQ(ARRIVALNODE, node->type);
  ASSERT_EQ(Time(5, 30, 3), node->time);
  node = transitNetwork.getNode(nodeIdsAtStop[13]);
  ASSERT_EQ(DEPARTURENODE, node->type);
  ASSERT_EQ(Time(5, 32, 0), node->time);
  // check that there is one edge edges to Guentzelstrasse
  edges = transitNetwork.getOutEdges(node->id);
  ASSERT_EQ(1, edges->size());
  ASSERT_EQ("Guentzelstrasse", transitNetwork.getStopName(
                             transitNetwork.getNode(edges->at(0).otherNodeId)->
                              stopId));
  for (size_t i = 0; i < transitNetwork._stops.size(); i++)
  {
    string stopName = transitNetwork._stops.at(i).stopName;
    vector<int>* stopNodes =
        &(transitNetwork._stops.at(i).nodeIdsSortedByTime);
    for (size_t j = 0; j < stopNodes->size(); j++)
    {
      int nodeId = stopNodes->at(j);
      node = transitNetwork.getNode(nodeId);
      ASSERT_EQ(stopName,
              transitNetwork._stops.at(node->stopId).stopName);
    }
  }


  // check that nodes have expected oredering (arrival transfer departure)
  // and that transfer noes always have edge to next transfer nodes,
  // except for last transfer nodes on that day

  for (size_t i = 0; i < transitNetwork.getNumNodes(); i++)
  {
    node = transitNetwork.getNode(i);
    if (i % 3 == 0)
    {
      ASSERT_EQ(ARRIVALNODE, node->type);
    }
    else if (i % 3 == 1)
    {
      ASSERT_EQ(TRANSFERNODE, node->type);
      // check edges if this transfernode is not the last transfernode of
      // that stop...
      const vector<int>& stopNodeIds = transitNetwork.
                                       getStopNodeIds(node->stopId);
      // they are sorted by time, so go backwards till you find
      // last transfer node at this stop
      NodeExpanded* nodeAtStop;
      size_t j = stopNodeIds.size() - 1;
      while (j > 0)
      {
        nodeAtStop = transitNetwork.getNode(stopNodeIds[j]);
        if (nodeAtStop->type == TRANSFERNODE)
          break;
        j--;
      }
      assert(j > 0);
      bool nodeIsLastTransferNodeAtStop = nodeAtStop->id == node->id;
      if (!nodeIsLastTransferNodeAtStop)
      {
        vector<Edge>* outEdges = transitNetwork.getOutEdges(i);
        ASSERT_GT(outEdges->size(), 0);
        NodeExpanded* otherNode;
        for (size_t j = 0; j < outEdges->size() - 1; j++)
        {
          otherNode = transitNetwork.getNode(outEdges->at(j).otherNodeId);
          ASSERT_EQ(DEPARTURENODE, otherNode->type);
        }
        otherNode = transitNetwork.getNode(outEdges->back().otherNodeId);
        ASSERT_EQ(TRANSFERNODE, otherNode->type);
      }
    }
    else if (i % 3 == 2)
    {
      ASSERT_EQ(DEPARTURENODE, node->type);
    }
  }
  // check whether the existing edges are correct
  vector<int> stopNodeIds;
  transitNetwork.getStopNodeIds("Eisenacher Strasse", &stopNodeIds);
  for (size_t i = 0; i < stopNodeIds.size(); i++)
  {
    int nodeId = stopNodeIds[i];
    NodeExpanded* node = transitNetwork.getNode(nodeId);
    ASSERT_EQ("Eisenacher Strasse",
            transitNetwork._stops.at(node->stopId).stopName);
    vector<Edge>* outEdges = transitNetwork.getOutEdges(nodeId);
    // should only have edges to itself or to bayerischer platz...
    for (size_t j = 0; j < outEdges->size(); j++)
    {
      Edge& nextEdge = outEdges->at(j);
      NodeExpanded* otherNode = transitNetwork.getNode(nextEdge.otherNodeId);
      if (node->type == ARRIVALNODE || node->type == TRANSFERNODE)
      {
        ASSERT_EQ("Eisenacher Strasse",
                transitNetwork._stops.at(otherNode->stopId).stopName);
      }
      else if (node->type == DEPARTURENODE)
      {
        ASSERT_EQ("Bayerischer Platz",
                  transitNetwork._stops.at(otherNode->stopId).stopName);
      }
    }
  }

  // check for berliner strasse because it has edges to guentzelstraße and to
  // blissestrasse..
  stopNodeIds.clear();
  transitNetwork.getStopNodeIds("Berliner Strasse", &stopNodeIds);
  for (size_t i = 0; i < stopNodeIds.size(); i++)
  {
    int nodeId = stopNodeIds[i];
    NodeExpanded* node = transitNetwork.getNode(nodeId);
    ASSERT_EQ("Berliner Strasse",
            transitNetwork._stops.at(node->stopId).stopName);

    vector<Edge>* outEdges = transitNetwork.getOutEdges(nodeId);
    // should only have edges to itself or to bayerischer platz...
    for (size_t j = 0; j < outEdges->size(); j++)
    {
      Edge& nextEdge = outEdges->at(j);
      NodeExpanded* otherNode = transitNetwork.getNode(nextEdge.otherNodeId);
      if (node->type == ARRIVALNODE || node->type == TRANSFERNODE)
      {
        ASSERT_EQ("Berliner Strasse",
                transitNetwork._stops.at(otherNode->stopId).stopName);
      }
      else if (node->type == DEPARTURENODE)
      {
        if (node->time.minute() == 35 && otherNode->time.minute() == 36)
          ASSERT_EQ("Blissestrasse",
                transitNetwork._stops.at(otherNode->stopId).stopName);
        else
          ASSERT_EQ("Guentzelstrasse",
                transitNetwork._stops.at(otherNode->stopId).stopName);
      }
    }
  }
}
